@@ -144,6 +144,10 @@ SpSubview<eT>::operator*=(const eT val)
   {
   arma_extra_debug_sigprint();
   
+  if(val == eT(0))  { (*this).zeros(); return *this; }
+  
+  if((n_elem == 0) || (n_nonzero == 0))  { return *this; }
+  
   m.sync_csc();
   m.invalidate_cache();
   
@@ -263,101 +267,33 @@ SpSubview<eT>::operator=(const Base<eT, T1>& in)
   {
   arma_extra_debug_sigprint();
   
+  if(is_same_type< T1, Gen<Mat<eT>, gen_zeros> >::yes)
+    {
+    const Proxy<T1> P(in.get_ref());
+    
+    arma_debug_assert_same_size(n_rows, n_cols, P.get_n_rows(), P.get_n_cols(), "insertion into sparse submatrix");
+    
+    (*this).zeros();
+    
+    return *this;
+    }
+  
+  if(is_same_type< T1, Gen<Mat<eT>, gen_eye> >::yes)
+    {
+    const Proxy<T1> P(in.get_ref());
+    
+    arma_debug_assert_same_size(n_rows, n_cols, P.get_n_rows(), P.get_n_cols(), "insertion into sparse submatrix");
+    
+    (*this).eye();
+    
+    return *this;
+    }
+  
   const quasi_unwrap<T1> U(in.get_ref());
   
   arma_debug_assert_same_size(n_rows, n_cols, U.M.n_rows, U.M.n_cols, "insertion into sparse submatrix");
   
-  if(n_elem == 0)  { return *this; }
-  
-  
-  // generate matrix B
-  
-  const eT*   U_M_memptr = U.M.memptr();
-  const uword U_M_n_elem = U.M.n_elem;
-  
-  uword B_n_nonzero = 0;
-  
-  for(uword i=0; i < U_M_n_elem; ++i)
-    {
-    B_n_nonzero += (U_M_memptr[i] != eT(0)) ? uword(1) : uword(0);
-    }
-  
-  SpMat<eT> B(arma_reserve_indicator(), m.n_rows, m.n_cols, B_n_nonzero);
-  
-  uword B_count = 0;
-  
-  for(uword col=0; col < U.M.n_cols; ++col)
-  for(uword row=0; row < U.M.n_rows; ++row)
-    {
-    const eT val = (*U_M_memptr);  U_M_memptr++;
-    
-    if(val != eT(0))
-      {
-      access::rw(B.values[B_count])      = val;
-      access::rw(B.row_indices[B_count]) = row + aux_row1;
-      access::rw(B.col_ptrs[col + aux_col1 + 1])++;
-      ++B_count;
-      }
-    }
-  
-  for(uword i=0; i < B.n_cols; ++i)
-    {
-    access::rw(B.col_ptrs[i + 1]) += B.col_ptrs[i];
-    }
-  
-  
-  if(n_nonzero == 0)
-    {
-    //// insert into submatrix
-    
-    //access::rw(m) = m + B;
-    spglue_merge::apply(access::rw(m), m, B);
-    }
-  else
-    {
-    // generate matrix A
-    
-    const uword row_start = aux_row1;
-    const uword col_start = aux_col1;
-    
-    const uword row_end   = aux_row1 + n_rows - 1;
-    const uword col_end   = aux_col1 + n_cols - 1;
-    
-    SpMat<eT> A(arma_reserve_indicator(), m.n_rows, m.n_cols, m.n_nonzero - n_nonzero);
-    
-    typename SpMat<eT>::const_iterator m_it     = m.begin();
-    typename SpMat<eT>::const_iterator m_it_end = m.end();
-    
-    uword A_count = 0;
-    
-    for(; m_it != m_it_end; ++m_it)
-      {
-      const uword m_it_row = m_it.row();
-      const uword m_it_col = m_it.col();
-      
-      const bool inside_box = ((m_it_row >= row_start) && (m_it_row <= row_end)) && ((m_it_col >= col_start) && (m_it_col <= col_end));
-      
-      if(inside_box == false)
-        {
-        access::rw(A.values[A_count])      = (*m_it);
-        access::rw(A.row_indices[A_count]) = m_it_row;
-        access::rw(A.col_ptrs[m_it_col + 1])++;
-        ++A_count;
-        }
-      }
-    
-    for(uword i=0; i < A.n_cols; ++i)
-      {
-      access::rw(A.col_ptrs[i + 1]) += A.col_ptrs[i];
-      }
-    
-    //// insert into submatrix
-    
-    //access::rw(m) = A + B;
-    spglue_merge::apply(access::rw(m), A, B);
-    }
-  
-  access::rw(n_nonzero) = B_n_nonzero;
+  spglue_merge::subview_merge(*this, U.M);
   
   return *this;
   }
@@ -477,89 +413,7 @@ SpSubview<eT>::operator_equ_common(const SpBase<eT, T1>& in)
   
   arma_debug_assert_same_size(n_rows, n_cols, U.M.n_rows, U.M.n_cols, "insertion into sparse submatrix");
   
-  if(n_elem == 0)  { return *this; }
-  
-  
-  // generate matrix B
-  
-  const uword U_M_n_nonzero = U.M.n_nonzero;
-  
-  SpMat<eT> B(arma_reserve_indicator(), m.n_rows, m.n_cols, U_M_n_nonzero);
-  
-  typename SpMat<eT>::const_iterator U_it = U.M.begin();
-  
-  for(uword i=0; i < U_M_n_nonzero; ++i)
-    {
-    access::rw(B.row_indices[i]) = U_it.row() + aux_row1;
-    access::rw(B.col_ptrs[U_it.col() + aux_col1 + 1])++;
-    ++U_it;
-    }
-  
-  for(uword i=0; i < B.n_cols; ++i)
-    {
-    access::rw(B.col_ptrs[i + 1]) += B.col_ptrs[i];
-    }
-  
-  const eT* B_values_orig = B.values;
-  
-  access::rw(B.values) = U.M.values;  // copy pointer instead of the entire array
-  
-  
-  if(n_nonzero == 0)
-    {
-    //// insert into submatrix
-    
-    //access::rw(m) = m + B;
-    spglue_merge::apply(access::rw(m), m, B);
-    }
-  else
-    {
-    // generate matrix A
-    
-    const uword row_start = aux_row1;
-    const uword col_start = aux_col1;
-    
-    const uword row_end   = aux_row1 + n_rows - 1;
-    const uword col_end   = aux_col1 + n_cols - 1;
-    
-    SpMat<eT> A(arma_reserve_indicator(), m.n_rows, m.n_cols, m.n_nonzero - n_nonzero);
-    
-    typename SpMat<eT>::const_iterator m_it     = m.begin();
-    typename SpMat<eT>::const_iterator m_it_end = m.end();
-    
-    uword A_count = 0;
-    
-    for(; m_it != m_it_end; ++m_it)
-      {
-      const uword m_it_row = m_it.row();
-      const uword m_it_col = m_it.col();
-      
-      const bool inside_box = ((m_it_row >= row_start) && (m_it_row <= row_end)) && ((m_it_col >= col_start) && (m_it_col <= col_end));
-      
-      if(inside_box == false)
-        {
-        access::rw(A.values[A_count])      = (*m_it);
-        access::rw(A.row_indices[A_count]) = m_it_row;
-        access::rw(A.col_ptrs[m_it_col + 1])++;
-        ++A_count;
-        }
-      }
-    
-    for(uword i=0; i < A.n_cols; ++i)
-      {
-      access::rw(A.col_ptrs[i + 1]) += A.col_ptrs[i];
-      }
-    
-    //// insert into submatrix
-    
-    //access::rw(m) = A + B;
-    spglue_merge::apply(access::rw(m), A, B);
-    }
-  
-  
-  access::rw(B.values) = B_values_orig;  // restore original pointer
-  
-  access::rw(n_nonzero) = U_M_n_nonzero;
+  spglue_merge::subview_merge(*this, U.M);
   
   return *this;
   }
@@ -886,6 +740,34 @@ SpSubview<eT>::replace(const eT old_val, const eT new_val)
 
 
 
+template<typename eT>
+inline
+void
+SpSubview<eT>::clean(const typename get_pod_type<eT>::result threshold)
+  {
+  arma_extra_debug_sigprint();
+  
+  if((n_elem == 0) || (n_nonzero == 0))  { return; }
+  
+  // TODO: replace with a more efficient implementation
+  
+  SpMat<eT> tmp(*this);
+  
+  tmp.clean(threshold);
+  
+  if(is_cx<eT>::yes)
+    {
+    (*this).operator=(tmp);
+    }
+  else
+  if(tmp.n_nonzero != n_nonzero)
+    {
+    (*this).operator=(tmp);
+    }
+  }
+
+
+
 template<typename eT>
 inline
 void
@@ -916,7 +798,52 @@ SpSubview<eT>::zeros()
   {
   arma_extra_debug_sigprint();
   
-  (*this).operator*=(eT(0));
+  if((n_elem == 0) || (n_nonzero == 0))  { return; }
+  
+  if((m.n_nonzero - n_nonzero) == 0)
+    {
+    access::rw(m).zeros();
+    access::rw(n_nonzero) = 0;
+    return;
+    }
+  
+  SpMat<eT> tmp(arma_reserve_indicator(), m.n_rows, m.n_cols, m.n_nonzero - n_nonzero);
+  
+  const uword sv_row_start = aux_row1;
+  const uword sv_col_start = aux_col1;
+  
+  const uword sv_row_end   = aux_row1 + n_rows - 1;
+  const uword sv_col_end   = aux_col1 + n_cols - 1;
+  
+  typename SpMat<eT>::const_iterator m_it     = m.begin();
+  typename SpMat<eT>::const_iterator m_it_end = m.end();
+  
+  uword tmp_count = 0;
+  
+  for(; m_it != m_it_end; ++m_it)
+    {
+    const uword m_it_row = m_it.row();
+    const uword m_it_col = m_it.col();
+    
+    const bool inside_box = ((m_it_row >= sv_row_start) && (m_it_row <= sv_row_end)) && ((m_it_col >= sv_col_start) && (m_it_col <= sv_col_end));
+    
+    if(inside_box == false)
+      {
+      access::rw(tmp.values[tmp_count])      = (*m_it);
+      access::rw(tmp.row_indices[tmp_count]) = m_it_row;
+      access::rw(tmp.col_ptrs[m_it_col + 1])++;
+      ++tmp_count;
+      }
+    }
+  
+  for(uword i=0; i < tmp.n_cols; ++i)
+    {
+    access::rw(tmp.col_ptrs[i + 1]) += tmp.col_ptrs[i];
+    }
+  
+  access::rw(m).steal_mem(tmp);
+  
+  access::rw(n_nonzero) = 0;
   }
 
 
@@ -1138,56 +1065,56 @@ SpSubview<eT>::is_vec() const
 
 template<typename eT>
 inline
-SpSubview<eT>
+SpSubview_row<eT>
 SpSubview<eT>::row(const uword row_num)
   {
   arma_extra_debug_sigprint();
-
+  
   arma_debug_check(row_num >= n_rows, "SpSubview::row(): out of bounds");
-
-  return submat(row_num, 0, row_num, n_cols - 1);
+  
+  return SpSubview_row<eT>(const_cast< SpMat<eT>& >(m), row_num + aux_row1, aux_col1, n_cols);
   }
 
 
 
 template<typename eT>
 inline
-const SpSubview<eT>
+const SpSubview_row<eT>
 SpSubview<eT>::row(const uword row_num) const
   {
   arma_extra_debug_sigprint();
-
+  
   arma_debug_check(row_num >= n_rows, "SpSubview::row(): out of bounds");
-
-  return submat(row_num, 0, row_num, n_cols - 1);
+  
+  return SpSubview_row<eT>(m, row_num + aux_row1, aux_col1, n_cols);
   }
 
 
 
 template<typename eT>
 inline
-SpSubview<eT>
+SpSubview_col<eT>
 SpSubview<eT>::col(const uword col_num)
   {
   arma_extra_debug_sigprint();
-
+  
   arma_debug_check(col_num >= n_cols, "SpSubview::col(): out of bounds");
-
-  return submat(0, col_num, n_rows - 1, col_num);
+  
+  return SpSubview_col<eT>(const_cast< SpMat<eT>& >(m), col_num + aux_col1, aux_row1, n_rows);
   }
 
 
 
 template<typename eT>
 inline
-const SpSubview<eT>
+const SpSubview_col<eT>
 SpSubview<eT>::col(const uword col_num) const
   {
   arma_extra_debug_sigprint();
-
+  
   arma_debug_check(col_num >= n_cols, "SpSubview::col(): out of bounds");
-
-  return submat(0, col_num, n_rows - 1, col_num);
+  
+  return SpSubview_col<eT>(m, col_num + aux_col1, aux_row1, n_rows);
   }
 
 
@@ -1695,69 +1622,256 @@ SpSubview<eT>::invalidate_cache() const
 
 
 
-/**
- * Sparse subview col
- *
+//
+//
+//
+
+
+
 template<typename eT>
 inline
-SpSubview_col<eT>::SpSubview_col(const Mat<eT>& in_m, const uword in_col)
+SpSubview_col<eT>::SpSubview_col(const SpMat<eT>& in_m, const uword in_col)
+  : SpSubview<eT>(in_m, 0, in_col, in_m.n_rows, 1)
   {
   arma_extra_debug_sigprint();
   }
 
+
+
 template<typename eT>
 inline
-SpSubview_col<eT>::SpSubview_col(Mat<eT>& in_m, const uword in_col)
+SpSubview_col<eT>::SpSubview_col(SpMat<eT>& in_m, const uword in_col)
+  : SpSubview<eT>(in_m, 0, in_col, in_m.n_rows, 1)
   {
   arma_extra_debug_sigprint();
   }
 
+
+
 template<typename eT>
 inline
-SpSubview_col<eT>::SpSubview_col(const Mat<eT>& in_m, const uword in_col, const uword in_row1, const uword in_n_rows)
+SpSubview_col<eT>::SpSubview_col(const SpMat<eT>& in_m, const uword in_col, const uword in_row1, const uword in_n_rows)
+  : SpSubview<eT>(in_m, in_row1, in_col, in_n_rows, 1)
   {
   arma_extra_debug_sigprint();
   }
 
-template<typename eT>
-inline
-SpSubview_col<eT>::SpSubview_col(Mat<eT>& in_m, const uword in_col, const uword in_row1, const uword in_n_rows)
-  {
-  arma_extra_debug_sigprint();
-  }
-*/
 
-/**
- * Sparse subview row
- *
+
 template<typename eT>
 inline
-SpSubview_row<eT>::SpSubview_row(const Mat<eT>& in_m, const uword in_row)
+SpSubview_col<eT>::SpSubview_col(SpMat<eT>& in_m, const uword in_col, const uword in_row1, const uword in_n_rows)
+  : SpSubview<eT>(in_m, in_row1, in_col, in_n_rows, 1)
   {
   arma_extra_debug_sigprint();
   }
 
+
+
 template<typename eT>
 inline
-SpSubview_row<eT>::SpSubview_row(Mat<eT>& in_m, const uword in_row)
+void
+SpSubview_col<eT>::operator=(const SpSubview<eT>& x)
+  {
+  arma_extra_debug_sigprint();
+  
+  SpSubview<eT>::operator=(x);
+  }
+
+
+
+template<typename eT>
+inline
+void
+SpSubview_col<eT>::operator=(const SpSubview_col<eT>& x)
+  {
+  arma_extra_debug_sigprint();
+  
+  SpSubview<eT>::operator=(x); // interprets 'SpSubview_col' as 'SpSubview'
+  }
+
+
+
+template<typename eT>
+template<typename T1>
+inline
+void
+SpSubview_col<eT>::operator=(const SpBase<eT,T1>& x)
+  {
+  arma_extra_debug_sigprint();
+  
+  SpSubview<eT>::operator=(x);
+  }
+
+
+
+template<typename eT>
+template<typename T1>
+inline
+void
+SpSubview_col<eT>::operator=(const Base<eT,T1>& x)
+  {
+  arma_extra_debug_sigprint();
+  
+  SpSubview<eT>::operator=(x);
+  }
+
+
+
+template<typename eT>
+inline
+const SpOp<SpSubview_col<eT>,spop_htrans>
+SpSubview_col<eT>::t() const
+  {
+  return SpOp<SpSubview_col<eT>,spop_htrans>(*this);
+  }
+
+
+
+template<typename eT>
+inline
+const SpOp<SpSubview_col<eT>,spop_htrans>
+SpSubview_col<eT>::ht() const
+  {
+  return SpOp<SpSubview_col<eT>,spop_htrans>(*this);
+  }
+
+
+
+template<typename eT>
+inline
+const SpOp<SpSubview_col<eT>,spop_strans>
+SpSubview_col<eT>::st() const
+  {
+  return SpOp<SpSubview_col<eT>,spop_strans>(*this);
+  }
+
+
+
+//
+//
+//
+
+
+
+template<typename eT>
+inline
+SpSubview_row<eT>::SpSubview_row(const SpMat<eT>& in_m, const uword in_row)
+  : SpSubview<eT>(in_m, in_row, 0, 1, in_m.n_cols)
   {
   arma_extra_debug_sigprint();
   }
 
+
+
 template<typename eT>
 inline
-SpSubview_row<eT>::SpSubview_row(const Mat<eT>& in_m, const uword in_row, const uword in_col1, const uword in_n_cols)
+SpSubview_row<eT>::SpSubview_row(SpMat<eT>& in_m, const uword in_row)
+  : SpSubview<eT>(in_m, in_row, 0, 1, in_m.n_cols)
   {
   arma_extra_debug_sigprint();
   }
 
+
+
 template<typename eT>
 inline
-SpSubview_row<eT>::SpSubview_row(Mat<eT>& in_m, const uword in_row, const uword in_col1, const uword in_n_cols)
+SpSubview_row<eT>::SpSubview_row(const SpMat<eT>& in_m, const uword in_row, const uword in_col1, const uword in_n_cols)
+  : SpSubview<eT>(in_m, in_row, in_col1, 1, in_n_cols)
   {
   arma_extra_debug_sigprint();
   }
-*/
+
+
+
+template<typename eT>
+inline
+SpSubview_row<eT>::SpSubview_row(SpMat<eT>& in_m, const uword in_row, const uword in_col1, const uword in_n_cols)
+  : SpSubview<eT>(in_m, in_row, in_col1, 1, in_n_cols)
+  {
+  arma_extra_debug_sigprint();
+  }
+
+
+
+template<typename eT>
+inline
+void
+SpSubview_row<eT>::operator=(const SpSubview<eT>& x)
+  {
+  arma_extra_debug_sigprint();
+  
+  SpSubview<eT>::operator=(x);
+  }
+
+
+
+template<typename eT>
+inline
+void
+SpSubview_row<eT>::operator=(const SpSubview_row<eT>& x)
+  {
+  arma_extra_debug_sigprint();
+  
+  SpSubview<eT>::operator=(x); // interprets 'SpSubview_row' as 'SpSubview'
+  }
+
+
+
+template<typename eT>
+template<typename T1>
+inline
+void
+SpSubview_row<eT>::operator=(const SpBase<eT,T1>& x)
+  {
+  arma_extra_debug_sigprint();
+  
+  SpSubview<eT>::operator=(x);
+  }
+
+
+
+template<typename eT>
+template<typename T1>
+inline
+void
+SpSubview_row<eT>::operator=(const Base<eT,T1>& x)
+  {
+  arma_extra_debug_sigprint();
+  
+  SpSubview<eT>::operator=(x);
+  }
+
+
+
+template<typename eT>
+inline
+const SpOp<SpSubview_row<eT>,spop_htrans>
+SpSubview_row<eT>::t() const
+  {
+  return SpOp<SpSubview_row<eT>,spop_htrans>(*this);
+  }
+
+
+
+template<typename eT>
+inline
+const SpOp<SpSubview_row<eT>,spop_htrans>
+SpSubview_row<eT>::ht() const
+  {
+  return SpOp<SpSubview_row<eT>,spop_htrans>(*this);
+  }
+
+
+
+template<typename eT>
+inline
+const SpOp<SpSubview_row<eT>,spop_strans>
+SpSubview_row<eT>::st() const
+  {
+  return SpOp<SpSubview_row<eT>,spop_strans>(*this);
+  }
+
 
 
 //! @}
